import { CommonsType, CommonsDate } from 'tscommons-core';
import { CommonsRestService, CommonsAuthorizedRestService, ECommonsAuthorizationMethod } from 'ngx-httpcommons-rest';
import { EAdamantineFieldType } from 'ngx-adamantinecommons-core';
import { __awaiter } from 'tslib';
import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AdamantineRestService extends CommonsRestService {
    /**
     * @param {?} fields
     * @return {?}
     */
    static buildFieldLookup(fields) {
        /** @type {?} */
        const map = new Map();
        for (const field of fields)
            map.set(field.name, field);
        return map;
    }
    /**
     * @param {?} incoming
     * @param {?} types
     * @return {?}
     */
    static parseData(incoming, types) {
        if (!CommonsType.isEncodedObject(incoming))
            throw new Error('Incoming data is not TEncodedObject');
        /** @type {?} */
        const encoded = (/** @type {?} */ (incoming));
        // don't use decodeObject, as it might try and parse string dates into dates. Do it manually here instead.
        /** @type {?} */
        const map = AdamantineRestService.buildFieldLookup(types);
        /** @type {?} */
        const data = {};
        for (const key of Object.keys(encoded)) {
            if (!map.has(key))
                throw new Error('No such type exists for the given key');
            if (encoded[key] === null)
                continue;
            /** @type {?} */
            const type = map.get(key);
            if (type === undefined)
                continue;
            switch (type.type) {
                case EAdamantineFieldType.SINGLE:
                case EAdamantineFieldType.MULTI:
                case EAdamantineFieldType.PHONE:
                case EAdamantineFieldType.EMAIL:
                case EAdamantineFieldType.URL:
                    data[key] = encoded[key].toString();
                    break;
                case EAdamantineFieldType.BOOLEAN:
                    data[key] = encoded[key] === true || [
                        'true', 'True', 'TRUE',
                        'yes', 'Yes', 'YES',
                        'on', 'On', 'ON',
                        '1'
                    ].includes(encoded[key].toString());
                    break;
                case EAdamantineFieldType.INTEGER:
                    data[key] = parseInt(encoded[key].toString(), 10);
                    break;
                case EAdamantineFieldType.FLOAT:
                    data[key] = parseFloat(encoded[key].toString());
                    break;
                case EAdamantineFieldType.DATE:
                    data[key] = CommonsDate.dmYToDate(encoded[key].toString());
                    break;
                case EAdamantineFieldType.TIME:
                    data[key] = CommonsDate.HiToDate(encoded[key].toString());
                    break;
                case EAdamantineFieldType.DATETIME:
                    data[key] = CommonsDate.dmYHiToDate(encoded[key].toString());
                    break;
                case EAdamantineFieldType.LISTSET:
                    data[key] = encoded[key].toString();
                    break;
                default:
                // allow fall-through so subclasses can handle additional custom types
            }
        }
        return data;
    }
    /**
     * @param {?} data
     * @param {?} types
     * @return {?}
     */
    static encodeData(data, types) {
        /** @type {?} */
        const map = AdamantineRestService.buildFieldLookup(types);
        /** @type {?} */
        const outgoing = {};
        for (const key of Object.keys(data)) {
            if (!map.has(key))
                throw new Error('No such type exists for the given key');
            if (data[key] === null) {
                outgoing[key] = null;
                continue;
            }
            /** @type {?} */
            const type = map.get(key);
            if (type === undefined)
                continue;
            switch (type.type) {
                case EAdamantineFieldType.SINGLE:
                case EAdamantineFieldType.MULTI:
                case EAdamantineFieldType.PHONE:
                case EAdamantineFieldType.EMAIL:
                case EAdamantineFieldType.URL:
                    outgoing[key] = data[key].toString();
                    break;
                case EAdamantineFieldType.BOOLEAN:
                    outgoing[key] = data[key] ? true : false;
                    break;
                case EAdamantineFieldType.INTEGER:
                    outgoing[key] = parseInt(data[key].toString(), 10);
                    break;
                case EAdamantineFieldType.FLOAT:
                    outgoing[key] = parseFloat(data[key].toString());
                    break;
                case EAdamantineFieldType.DATE:
                    if (!CommonsType.isDate(data[key]))
                        throw new Error('Value is not a date');
                    outgoing[key] = CommonsDate.dateTodmY((/** @type {?} */ (data[key])));
                    break;
                case EAdamantineFieldType.TIME:
                    if (!CommonsType.isDate(data[key]))
                        throw new Error('Value is not a date');
                    outgoing[key] = CommonsDate.dateToHi((/** @type {?} */ (data[key])));
                    break;
                case EAdamantineFieldType.DATETIME:
                    if (!CommonsType.isDate(data[key]))
                        throw new Error('Value is not a date');
                    outgoing[key] = CommonsDate.dateTodmYHi((/** @type {?} */ (data[key])));
                    break;
                case EAdamantineFieldType.LISTSET:
                    outgoing[key] = data[key].toString();
                    break;
                default:
                // allow fall-through so subclasses can handle additional custom types
            }
        }
        return CommonsType.encodePropertyObject(outgoing);
    }
    /**
     * @param {?} rootURL
     * @param {?} http
     */
    constructor(rootURL, http) {
        super(rootURL, http);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AdamantineSessionRestService extends AdamantineRestService {
    /**
     * @param {?} rootURL
     * @param {?} http
     */
    constructor(rootURL, http) {
        super(rootURL, http);
    }
    /**
     * @param {?} uid
     * @return {?}
     */
    setSession(uid) {
        this.session = uid;
    }
    /**
     * @return {?}
     */
    clearSession() {
        this.session = undefined;
    }
    /**
     * @private
     * @param {?} script
     * @return {?}
     */
    extendScript(script) {
        if (this.session === undefined)
            return script;
        /** @type {?} */
        const char = -1 !== script.indexOf('?') ? '&' : '?';
        return `${script}${char}session=${this.session}`;
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @return {?}
     */
    internalGet(script, params, maxReattempts) {
        const _super = Object.create(null, {
            internalGet: { get: () => super.internalGet }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalGet.call(this, this.extendScript(script), params, maxReattempts);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    internalPost(script, body, params) {
        const _super = Object.create(null, {
            internalPost: { get: () => super.internalPost }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalPost.call(this, this.extendScript(script), body, params);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    internalPut(script, body, params) {
        const _super = Object.create(null, {
            internalPut: { get: () => super.internalPut }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalPut.call(this, this.extendScript(script), body, params);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    internalPatch(script, body, params) {
        const _super = Object.create(null, {
            internalPatch: { get: () => super.internalPatch }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalPatch.call(this, this.extendScript(script), body, params);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @return {?}
     */
    internalDelete(script, params) {
        const _super = Object.create(null, {
            internalDelete: { get: () => super.internalDelete }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalDelete.call(this, this.extendScript(script), params);
        });
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineSessionRestService.prototype.session;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AdamantineExternalKeyRestService extends AdamantineRestService {
    /**
     * @param {?} rootURL
     * @param {?} http
     */
    constructor(rootURL, http) {
        super(rootURL, http);
        this.externalKey = '';
    }
    /**
     * @param {?} key
     * @return {?}
     */
    setExternalKey(key) {
        this.externalKey = key;
    }
    /**
     * @private
     * @param {?} script
     * @return {?}
     */
    extendScript(script) {
        /** @type {?} */
        const char = -1 !== script.indexOf('?') ? '&' : '?';
        return `${script}${char}key=${this.externalKey}`;
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @return {?}
     */
    internalGet(script, params, maxReattempts) {
        const _super = Object.create(null, {
            internalGet: { get: () => super.internalGet }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalGet.call(this, this.extendScript(script), params, maxReattempts);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    internalPost(script, body, params) {
        const _super = Object.create(null, {
            internalPost: { get: () => super.internalPost }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalPost.call(this, this.extendScript(script), body, params);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    internalPut(script, body, params) {
        const _super = Object.create(null, {
            internalPut: { get: () => super.internalPut }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalPut.call(this, this.extendScript(script), body, params);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    internalPatch(script, body, params) {
        const _super = Object.create(null, {
            internalPatch: { get: () => super.internalPatch }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalPatch.call(this, this.extendScript(script), body, params);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @return {?}
     */
    internalDelete(script, params) {
        const _super = Object.create(null, {
            internalDelete: { get: () => super.internalDelete }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return yield _super.internalDelete.call(this, this.extendScript(script), params);
        });
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineExternalKeyRestService.prototype.externalKey;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AdamantineAuthorizedRestService extends CommonsAuthorizedRestService {
    // can't do multiple inheritence in AdamantineRestService, so have to do pass-through to static methods here
    /**
     * @param {?} fields
     * @return {?}
     */
    static buildFieldLookup(fields) {
        return AdamantineRestService.buildFieldLookup(fields);
    }
    /**
     * @param {?} incoming
     * @param {?} types
     * @return {?}
     */
    static parseData(incoming, types) {
        return AdamantineRestService.parseData(incoming, types);
    }
    /**
     * @param {?} data
     * @param {?} types
     * @return {?}
     */
    static encodeData(data, types) {
        return AdamantineRestService.encodeData(data, types);
    }
    /**
     * @param {?} rootURL
     * @param {?} http
     */
    constructor(rootURL, http) {
        super(rootURL, http);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AdamantineKeyAuthorizedRestService extends AdamantineAuthorizedRestService {
    /**
     * @param {?} rootURL
     * @param {?} http
     */
    constructor(rootURL, http) {
        super(rootURL, http);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    setKey(key) {
        this.setAuthorization(ECommonsAuthorizationMethod.KEY, key);
    }
    /**
     * @return {?}
     */
    clearKey() {
        this.setAuthorization(undefined, undefined);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AdamantineSessionAuthorizedRestService extends AdamantineAuthorizedRestService {
    /**
     * @param {?} rootURL
     * @param {?} http
     */
    constructor(rootURL, http) {
        super(rootURL, http);
    }
    /**
     * @param {?} uid
     * @return {?}
     */
    setSession(uid) {
        this.setAuthorization(ECommonsAuthorizationMethod.SESSION, uid);
    }
    /**
     * @return {?}
     */
    clearSession() {
        this.setAuthorization(undefined, undefined);
    }
    /**
     * @return {?}
     */
    validate() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                /** @type {?} */
                const rest = yield this.get(`sessions/validate`);
                CommonsType.assertBoolean(rest);
                return rest === true;
            }
            catch (e) {
                return false;
            }
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Prefers sessions over keys, but falls back to keys on the absence of a session
/**
 * @abstract
 */
class AdamantineHybridAuthorizedRestService extends AdamantineAuthorizedRestService {
    /**
     * @param {?} rootURL
     * @param {?} http
     */
    constructor(rootURL, http) {
        super(rootURL, http);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    setKey(key) {
        this.key = key;
    }
    /**
     * @return {?}
     */
    clearKey() {
        this.key = undefined;
    }
    /**
     * @param {?} uid
     * @return {?}
     */
    setSession(uid) {
        this.session = uid;
    }
    /**
     * @return {?}
     */
    clearSession() {
        this.session = undefined;
    }
    /**
     * @return {?}
     */
    hasSession() {
        return this.session !== undefined;
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @param {?=} headers
     * @return {?}
     */
    internalGet(script, params, maxReattempts, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let method;
            /** @type {?} */
            let value;
            if (this.session !== undefined) {
                method = ECommonsAuthorizationMethod.SESSION;
                value = this.session;
            }
            else if (this.key !== undefined) {
                method = ECommonsAuthorizationMethod.KEY;
                value = this.key;
            }
            return yield this.fixedAuthorizationGet(method, value, script, params, maxReattempts, headers);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    internalPost(script, body, params, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let method;
            /** @type {?} */
            let value;
            if (this.session !== undefined) {
                method = ECommonsAuthorizationMethod.SESSION;
                value = this.session;
            }
            else if (this.key !== undefined) {
                method = ECommonsAuthorizationMethod.KEY;
                value = this.key;
            }
            return yield this.fixedAuthorizationPost(method, value, script, body, params, headers);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    internalPut(script, body, params, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let method;
            /** @type {?} */
            let value;
            if (this.session !== undefined) {
                method = ECommonsAuthorizationMethod.SESSION;
                value = this.session;
            }
            else if (this.key !== undefined) {
                method = ECommonsAuthorizationMethod.KEY;
                value = this.key;
            }
            return yield this.fixedAuthorizationPut(method, value, script, body, params, headers);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    internalPatch(script, body, params, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let method;
            /** @type {?} */
            let value;
            if (this.session !== undefined) {
                method = ECommonsAuthorizationMethod.SESSION;
                value = this.session;
            }
            else if (this.key !== undefined) {
                method = ECommonsAuthorizationMethod.KEY;
                value = this.key;
            }
            return yield this.fixedAuthorizationPatch(method, value, script, body, params, headers);
        });
    }
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    internalDelete(script, params, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let method;
            /** @type {?} */
            let value;
            if (this.session !== undefined) {
                method = ECommonsAuthorizationMethod.SESSION;
                value = this.session;
            }
            else if (this.key !== undefined) {
                method = ECommonsAuthorizationMethod.KEY;
                value = this.key;
            }
            return yield this.fixedAuthorizationDelete(method, value, script, params, headers);
        });
    }
    /**
     * @return {?}
     */
    validate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.hasSession())
                return false;
            try {
                /** @type {?} */
                const rest = yield this.get(`sessions/validate`);
                CommonsType.assertBoolean(rest);
                return rest === true;
            }
            catch (e) {
                return false;
            }
        });
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineHybridAuthorizedRestService.prototype.key;
    /**
     * @type {?}
     * @private
     */
    AdamantineHybridAuthorizedRestService.prototype.session;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxAdamantineCommonsRestModule {
}
NgxAdamantineCommonsRestModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: []
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AdamantineAuthorizedRestService, AdamantineExternalKeyRestService, AdamantineHybridAuthorizedRestService, AdamantineKeyAuthorizedRestService, AdamantineRestService, AdamantineSessionAuthorizedRestService, AdamantineSessionRestService, NgxAdamantineCommonsRestModule };
//# sourceMappingURL=ngx-adamantinecommons-rest.js.map
