import { __extends, __values, __awaiter, __generator } from 'tslib';
import { CommonsType, CommonsDate } from 'tscommons-core';
import { CommonsRestService, CommonsAuthorizedRestService, ECommonsAuthorizationMethod } from 'ngx-httpcommons-rest';
import { EAdamantineFieldType } from 'ngx-adamantinecommons-core';
import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var  /**
 * @abstract
 */
AdamantineRestService = /** @class */ (function (_super) {
    __extends(AdamantineRestService, _super);
    function AdamantineRestService(rootURL, http) {
        return _super.call(this, rootURL, http) || this;
    }
    /**
     * @param {?} fields
     * @return {?}
     */
    AdamantineRestService.buildFieldLookup = /**
     * @param {?} fields
     * @return {?}
     */
    function (fields) {
        var e_1, _a;
        /** @type {?} */
        var map = new Map();
        try {
            for (var fields_1 = __values(fields), fields_1_1 = fields_1.next(); !fields_1_1.done; fields_1_1 = fields_1.next()) {
                var field = fields_1_1.value;
                map.set(field.name, field);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (fields_1_1 && !fields_1_1.done && (_a = fields_1.return)) _a.call(fields_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return map;
    };
    /**
     * @param {?} incoming
     * @param {?} types
     * @return {?}
     */
    AdamantineRestService.parseData = /**
     * @param {?} incoming
     * @param {?} types
     * @return {?}
     */
    function (incoming, types) {
        var e_2, _a;
        if (!CommonsType.isEncodedObject(incoming))
            throw new Error('Incoming data is not TEncodedObject');
        /** @type {?} */
        var encoded = (/** @type {?} */ (incoming));
        // don't use decodeObject, as it might try and parse string dates into dates. Do it manually here instead.
        /** @type {?} */
        var map = AdamantineRestService.buildFieldLookup(types);
        /** @type {?} */
        var data = {};
        try {
            for (var _b = __values(Object.keys(encoded)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var key = _c.value;
                if (!map.has(key))
                    throw new Error('No such type exists for the given key');
                if (encoded[key] === null)
                    continue;
                /** @type {?} */
                var type = map.get(key);
                if (type === undefined)
                    continue;
                switch (type.type) {
                    case EAdamantineFieldType.SINGLE:
                    case EAdamantineFieldType.MULTI:
                    case EAdamantineFieldType.PHONE:
                    case EAdamantineFieldType.EMAIL:
                    case EAdamantineFieldType.URL:
                        data[key] = encoded[key].toString();
                        break;
                    case EAdamantineFieldType.BOOLEAN:
                        data[key] = encoded[key] === true || [
                            'true', 'True', 'TRUE',
                            'yes', 'Yes', 'YES',
                            'on', 'On', 'ON',
                            '1'
                        ].includes(encoded[key].toString());
                        break;
                    case EAdamantineFieldType.INTEGER:
                        data[key] = parseInt(encoded[key].toString(), 10);
                        break;
                    case EAdamantineFieldType.FLOAT:
                        data[key] = parseFloat(encoded[key].toString());
                        break;
                    case EAdamantineFieldType.DATE:
                        data[key] = CommonsDate.dmYToDate(encoded[key].toString());
                        break;
                    case EAdamantineFieldType.TIME:
                        data[key] = CommonsDate.HiToDate(encoded[key].toString());
                        break;
                    case EAdamantineFieldType.DATETIME:
                        data[key] = CommonsDate.dmYHiToDate(encoded[key].toString());
                        break;
                    case EAdamantineFieldType.LISTSET:
                        data[key] = encoded[key].toString();
                        break;
                    default:
                    // allow fall-through so subclasses can handle additional custom types
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return data;
    };
    /**
     * @param {?} data
     * @param {?} types
     * @return {?}
     */
    AdamantineRestService.encodeData = /**
     * @param {?} data
     * @param {?} types
     * @return {?}
     */
    function (data, types) {
        var e_3, _a;
        /** @type {?} */
        var map = AdamantineRestService.buildFieldLookup(types);
        /** @type {?} */
        var outgoing = {};
        try {
            for (var _b = __values(Object.keys(data)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var key = _c.value;
                if (!map.has(key))
                    throw new Error('No such type exists for the given key');
                if (data[key] === null) {
                    outgoing[key] = null;
                    continue;
                }
                /** @type {?} */
                var type = map.get(key);
                if (type === undefined)
                    continue;
                switch (type.type) {
                    case EAdamantineFieldType.SINGLE:
                    case EAdamantineFieldType.MULTI:
                    case EAdamantineFieldType.PHONE:
                    case EAdamantineFieldType.EMAIL:
                    case EAdamantineFieldType.URL:
                        outgoing[key] = data[key].toString();
                        break;
                    case EAdamantineFieldType.BOOLEAN:
                        outgoing[key] = data[key] ? true : false;
                        break;
                    case EAdamantineFieldType.INTEGER:
                        outgoing[key] = parseInt(data[key].toString(), 10);
                        break;
                    case EAdamantineFieldType.FLOAT:
                        outgoing[key] = parseFloat(data[key].toString());
                        break;
                    case EAdamantineFieldType.DATE:
                        if (!CommonsType.isDate(data[key]))
                            throw new Error('Value is not a date');
                        outgoing[key] = CommonsDate.dateTodmY((/** @type {?} */ (data[key])));
                        break;
                    case EAdamantineFieldType.TIME:
                        if (!CommonsType.isDate(data[key]))
                            throw new Error('Value is not a date');
                        outgoing[key] = CommonsDate.dateToHi((/** @type {?} */ (data[key])));
                        break;
                    case EAdamantineFieldType.DATETIME:
                        if (!CommonsType.isDate(data[key]))
                            throw new Error('Value is not a date');
                        outgoing[key] = CommonsDate.dateTodmYHi((/** @type {?} */ (data[key])));
                        break;
                    case EAdamantineFieldType.LISTSET:
                        outgoing[key] = data[key].toString();
                        break;
                    default:
                    // allow fall-through so subclasses can handle additional custom types
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return CommonsType.encodePropertyObject(outgoing);
    };
    return AdamantineRestService;
}(CommonsRestService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var  /**
 * @abstract
 */
AdamantineSessionRestService = /** @class */ (function (_super) {
    __extends(AdamantineSessionRestService, _super);
    function AdamantineSessionRestService(rootURL, http) {
        return _super.call(this, rootURL, http) || this;
    }
    /**
     * @param {?} uid
     * @return {?}
     */
    AdamantineSessionRestService.prototype.setSession = /**
     * @param {?} uid
     * @return {?}
     */
    function (uid) {
        this.session = uid;
    };
    /**
     * @return {?}
     */
    AdamantineSessionRestService.prototype.clearSession = /**
     * @return {?}
     */
    function () {
        this.session = undefined;
    };
    /**
     * @private
     * @param {?} script
     * @return {?}
     */
    AdamantineSessionRestService.prototype.extendScript = /**
     * @private
     * @param {?} script
     * @return {?}
     */
    function (script) {
        if (this.session === undefined)
            return script;
        /** @type {?} */
        var char = -1 !== script.indexOf('?') ? '&' : '?';
        return "" + script + char + "session=" + this.session;
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @return {?}
     */
    AdamantineSessionRestService.prototype.internalGet = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @return {?}
     */
    function (script, params, maxReattempts) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalGet.call(this, this.extendScript(script), params, maxReattempts)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    AdamantineSessionRestService.prototype.internalPost = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    function (script, body, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalPost.call(this, this.extendScript(script), body, params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    AdamantineSessionRestService.prototype.internalPut = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    function (script, body, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalPut.call(this, this.extendScript(script), body, params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    AdamantineSessionRestService.prototype.internalPatch = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    function (script, body, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalPatch.call(this, this.extendScript(script), body, params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @return {?}
     */
    AdamantineSessionRestService.prototype.internalDelete = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @return {?}
     */
    function (script, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalDelete.call(this, this.extendScript(script), params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    return AdamantineSessionRestService;
}(AdamantineRestService));
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineSessionRestService.prototype.session;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var  /**
 * @abstract
 */
AdamantineExternalKeyRestService = /** @class */ (function (_super) {
    __extends(AdamantineExternalKeyRestService, _super);
    function AdamantineExternalKeyRestService(rootURL, http) {
        var _this = _super.call(this, rootURL, http) || this;
        _this.externalKey = '';
        return _this;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    AdamantineExternalKeyRestService.prototype.setExternalKey = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        this.externalKey = key;
    };
    /**
     * @private
     * @param {?} script
     * @return {?}
     */
    AdamantineExternalKeyRestService.prototype.extendScript = /**
     * @private
     * @param {?} script
     * @return {?}
     */
    function (script) {
        /** @type {?} */
        var char = -1 !== script.indexOf('?') ? '&' : '?';
        return "" + script + char + "key=" + this.externalKey;
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @return {?}
     */
    AdamantineExternalKeyRestService.prototype.internalGet = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @return {?}
     */
    function (script, params, maxReattempts) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalGet.call(this, this.extendScript(script), params, maxReattempts)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    AdamantineExternalKeyRestService.prototype.internalPost = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    function (script, body, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalPost.call(this, this.extendScript(script), body, params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    AdamantineExternalKeyRestService.prototype.internalPut = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    function (script, body, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalPut.call(this, this.extendScript(script), body, params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    AdamantineExternalKeyRestService.prototype.internalPatch = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @return {?}
     */
    function (script, body, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalPatch.call(this, this.extendScript(script), body, params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @return {?}
     */
    AdamantineExternalKeyRestService.prototype.internalDelete = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @return {?}
     */
    function (script, params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _super.prototype.internalDelete.call(this, this.extendScript(script), params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    return AdamantineExternalKeyRestService;
}(AdamantineRestService));
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineExternalKeyRestService.prototype.externalKey;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var  /**
 * @abstract
 */
AdamantineAuthorizedRestService = /** @class */ (function (_super) {
    __extends(AdamantineAuthorizedRestService, _super);
    function AdamantineAuthorizedRestService(rootURL, http) {
        return _super.call(this, rootURL, http) || this;
    }
    // can't do multiple inheritence in AdamantineRestService, so have to do pass-through to static methods here
    // can't do multiple inheritence in AdamantineRestService, so have to do pass-through to static methods here
    /**
     * @param {?} fields
     * @return {?}
     */
    AdamantineAuthorizedRestService.buildFieldLookup = 
    // can't do multiple inheritence in AdamantineRestService, so have to do pass-through to static methods here
    /**
     * @param {?} fields
     * @return {?}
     */
    function (fields) {
        return AdamantineRestService.buildFieldLookup(fields);
    };
    /**
     * @param {?} incoming
     * @param {?} types
     * @return {?}
     */
    AdamantineAuthorizedRestService.parseData = /**
     * @param {?} incoming
     * @param {?} types
     * @return {?}
     */
    function (incoming, types) {
        return AdamantineRestService.parseData(incoming, types);
    };
    /**
     * @param {?} data
     * @param {?} types
     * @return {?}
     */
    AdamantineAuthorizedRestService.encodeData = /**
     * @param {?} data
     * @param {?} types
     * @return {?}
     */
    function (data, types) {
        return AdamantineRestService.encodeData(data, types);
    };
    return AdamantineAuthorizedRestService;
}(CommonsAuthorizedRestService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var  /**
 * @abstract
 */
AdamantineKeyAuthorizedRestService = /** @class */ (function (_super) {
    __extends(AdamantineKeyAuthorizedRestService, _super);
    function AdamantineKeyAuthorizedRestService(rootURL, http) {
        return _super.call(this, rootURL, http) || this;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    AdamantineKeyAuthorizedRestService.prototype.setKey = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        this.setAuthorization(ECommonsAuthorizationMethod.KEY, key);
    };
    /**
     * @return {?}
     */
    AdamantineKeyAuthorizedRestService.prototype.clearKey = /**
     * @return {?}
     */
    function () {
        this.setAuthorization(undefined, undefined);
    };
    return AdamantineKeyAuthorizedRestService;
}(AdamantineAuthorizedRestService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var  /**
 * @abstract
 */
AdamantineSessionAuthorizedRestService = /** @class */ (function (_super) {
    __extends(AdamantineSessionAuthorizedRestService, _super);
    function AdamantineSessionAuthorizedRestService(rootURL, http) {
        return _super.call(this, rootURL, http) || this;
    }
    /**
     * @param {?} uid
     * @return {?}
     */
    AdamantineSessionAuthorizedRestService.prototype.setSession = /**
     * @param {?} uid
     * @return {?}
     */
    function (uid) {
        this.setAuthorization(ECommonsAuthorizationMethod.SESSION, uid);
    };
    /**
     * @return {?}
     */
    AdamantineSessionAuthorizedRestService.prototype.clearSession = /**
     * @return {?}
     */
    function () {
        this.setAuthorization(undefined, undefined);
    };
    /**
     * @return {?}
     */
    AdamantineSessionAuthorizedRestService.prototype.validate = /**
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var rest, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.get("sessions/validate")];
                    case 1:
                        rest = _a.sent();
                        CommonsType.assertBoolean(rest);
                        return [2 /*return*/, rest === true];
                    case 2:
                        e_1 = _a.sent();
                        return [2 /*return*/, false];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    return AdamantineSessionAuthorizedRestService;
}(AdamantineAuthorizedRestService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Prefers sessions over keys, but falls back to keys on the absence of a session
/**
 * @abstract
 */
var  
// Prefers sessions over keys, but falls back to keys on the absence of a session
/**
 * @abstract
 */
AdamantineHybridAuthorizedRestService = /** @class */ (function (_super) {
    __extends(AdamantineHybridAuthorizedRestService, _super);
    function AdamantineHybridAuthorizedRestService(rootURL, http) {
        return _super.call(this, rootURL, http) || this;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.setKey = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        this.key = key;
    };
    /**
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.clearKey = /**
     * @return {?}
     */
    function () {
        this.key = undefined;
    };
    /**
     * @param {?} uid
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.setSession = /**
     * @param {?} uid
     * @return {?}
     */
    function (uid) {
        this.session = uid;
    };
    /**
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.clearSession = /**
     * @return {?}
     */
    function () {
        this.session = undefined;
    };
    /**
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.hasSession = /**
     * @return {?}
     */
    function () {
        return this.session !== undefined;
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @param {?=} headers
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.internalGet = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxReattempts
     * @param {?=} headers
     * @return {?}
     */
    function (script, params, maxReattempts, headers) {
        return __awaiter(this, void 0, void 0, function () {
            var method, value;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.session !== undefined) {
                            method = ECommonsAuthorizationMethod.SESSION;
                            value = this.session;
                        }
                        else if (this.key !== undefined) {
                            method = ECommonsAuthorizationMethod.KEY;
                            value = this.key;
                        }
                        return [4 /*yield*/, this.fixedAuthorizationGet(method, value, script, params, maxReattempts, headers)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.internalPost = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    function (script, body, params, headers) {
        return __awaiter(this, void 0, void 0, function () {
            var method, value;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.session !== undefined) {
                            method = ECommonsAuthorizationMethod.SESSION;
                            value = this.session;
                        }
                        else if (this.key !== undefined) {
                            method = ECommonsAuthorizationMethod.KEY;
                            value = this.key;
                        }
                        return [4 /*yield*/, this.fixedAuthorizationPost(method, value, script, body, params, headers)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.internalPut = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    function (script, body, params, headers) {
        return __awaiter(this, void 0, void 0, function () {
            var method, value;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.session !== undefined) {
                            method = ECommonsAuthorizationMethod.SESSION;
                            value = this.session;
                        }
                        else if (this.key !== undefined) {
                            method = ECommonsAuthorizationMethod.KEY;
                            value = this.key;
                        }
                        return [4 /*yield*/, this.fixedAuthorizationPut(method, value, script, body, params, headers)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.internalPatch = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?} body
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    function (script, body, params, headers) {
        return __awaiter(this, void 0, void 0, function () {
            var method, value;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.session !== undefined) {
                            method = ECommonsAuthorizationMethod.SESSION;
                            value = this.session;
                        }
                        else if (this.key !== undefined) {
                            method = ECommonsAuthorizationMethod.KEY;
                            value = this.key;
                        }
                        return [4 /*yield*/, this.fixedAuthorizationPatch(method, value, script, body, params, headers)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.internalDelete = /**
     * @protected
     * @template T
     * @param {?} script
     * @param {?=} params
     * @param {?=} headers
     * @return {?}
     */
    function (script, params, headers) {
        return __awaiter(this, void 0, void 0, function () {
            var method, value;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.session !== undefined) {
                            method = ECommonsAuthorizationMethod.SESSION;
                            value = this.session;
                        }
                        else if (this.key !== undefined) {
                            method = ECommonsAuthorizationMethod.KEY;
                            value = this.key;
                        }
                        return [4 /*yield*/, this.fixedAuthorizationDelete(method, value, script, params, headers)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @return {?}
     */
    AdamantineHybridAuthorizedRestService.prototype.validate = /**
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var rest, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.hasSession())
                            return [2 /*return*/, false];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.get("sessions/validate")];
                    case 2:
                        rest = _a.sent();
                        CommonsType.assertBoolean(rest);
                        return [2 /*return*/, rest === true];
                    case 3:
                        e_1 = _a.sent();
                        return [2 /*return*/, false];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    return AdamantineHybridAuthorizedRestService;
}(AdamantineAuthorizedRestService));
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineHybridAuthorizedRestService.prototype.key;
    /**
     * @type {?}
     * @private
     */
    AdamantineHybridAuthorizedRestService.prototype.session;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxAdamantineCommonsRestModule = /** @class */ (function () {
    function NgxAdamantineCommonsRestModule() {
    }
    NgxAdamantineCommonsRestModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule
                    ],
                    declarations: []
                },] }
    ];
    return NgxAdamantineCommonsRestModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AdamantineAuthorizedRestService, AdamantineExternalKeyRestService, AdamantineHybridAuthorizedRestService, AdamantineKeyAuthorizedRestService, AdamantineRestService, AdamantineSessionAuthorizedRestService, AdamantineSessionRestService, NgxAdamantineCommonsRestModule };
//# sourceMappingURL=ngx-adamantinecommons-rest.js.map
